﻿export default function DetectSignature(submitterUserId) {
	//select ticket body + all email-comments from that user
	let elements = document.querySelectorAll(`#body, .commentDiv[data-userid="${submitterUserId}"] .ticketComment:not(.systemcomment).fromemail .commentBody > span`);
	if (elements.length < 2) return;

	let texts = Array.from(elements)
		.slice(0, 10) //take first 10 elements for performance
		.map(el => el.innerHTML.trimEnd());

	var unmodifiedText = texts[0];

	//remove the old "<i class='signStart'>" that came from server
	texts = texts.map(t => t.replace("<i class='signStart'></i>", ""));

	//normalize elements - trim ending whitespaces, divs and BR's
	let endsWithWhiteSpaceRegex = new RegExp("\\s$");
	for (let i = 0; i < texts.length; i++) {
		let txt = texts[i];
		while (txt.endsWith("</div>") || txt.endsWith("<br>") || endsWithWhiteSpaceRegex.test(txt)) {
			if (endsWithWhiteSpaceRegex.test(txt)) txt = txt.slice(0, -1);
			else if (txt.endsWith("</div>")) txt = txt.slice(0, -6);
			else if (txt.endsWith("<br>")) txt = txt.slice(0, -4);
		}
		texts[i] = txt;
	}

	//finding common suffix is same as "reverse + find common PREFIX"
	let reversedTexts = texts.map(reverseString); //reverseString is a function
	let len = findCommonPrefixLength(reversedTexts); //find common prefix length

	if (len < 20) return; //signature too short
	if (texts[0].length - len < 11) return; //text before signature is too short

	//at this popint, the common part starts "len" chars from the end, let's proceed

	//are we between HTML brackets?
	if (isPositionBetweenHtmlBrackets(texts[0], texts[0].length - len)) {
		let bracketpos = texts[0].indexOf('>', texts[0].length - len) + 1; //shift to the next ">"
		len = texts[0].length - bracketpos;
	}

	//now recalc len to compensate for normalizing
	len = unmodifiedText - (texts[0] - len);

	//modify HTML - add special marker where sig. starts
	for (let i = 0; i < elements.length; i++) {
		elements[i].innerHTML = elements[i].innerHTML.trimEnd().insertAtEnd(len, "<i class='signStart'></i>");
	}

	HideSignatures(); //this is copied from ViewTicket.js
}

function reverseString(str) {
	return str.split('').reduce((reversed, character) => character + reversed, '');
}

function findCommonPrefixLength(strings) {
	if (strings.length < 2) {
		return 0;
	}

	// Sort the array to make sure the shortest string is at the beginning, then find prefix between first and last string
	strings.sort();

	const firstString = strings[0];
	const lastString = strings[strings.length - 1];

	const minLength = Math.min(firstString.length, lastString.length);

	for (let i = 0; i < minLength; i++) {
		if (firstString[i] === lastString[i]) {
			continue;
		} else {
			return i;
		}
	}

	return minLength;
}

function isPositionBetweenHtmlBrackets(input, position) {
	if (position >= input.length) return false;

	for (let i = position; i < input.length; i++) //first run (to the right)
	{
		if (input[i] == '<') return false;
		if (input[i] == '>') return true;
	}

	return false;
}

function HideSignatures() {
	if ($(".signStart").length == 0) return;
	if ($(".signatureTag").length != 0) return; //already processed

	$(".signStart").each(function (i, el) {
		HideSiblings(el); //mark all NEXT siblings with "signatureTag"

		//walk up to root and do the same
		var parent = el.parentElement;
		while (parent != null && !parent.classList.contains("commentBody") && parent.id != "body") {
			HideSiblings(parent);
			parent = parent.parentElement;
		}

		//marks all siblings as "signature"
		function HideSiblings(el) {
			while (el.nextSibling) { //mark all NEXT siblings with "signatureTag"
				el = el.nextSibling;
				if (el.nodeType == Node.TEXT_NODE) { //if text -  surround with span
					if (el.nodeValue && el.nodeValue.length)
						el = $("<span class='signatureTag'>" + el.nodeValue + "</span>").replaceAll(el)[0];
				}
				else
					el.classList.add("signatureTag");
			}
		}

		$(el).before("<br><button title='Show signature' class='signatureToggle editButton graybutton' type='button'>•••</button>");
	});

	$(".signatureToggle").click(function (el) {
		var scopeEl = $(this).parents(".commentBody, #body");
		$(".signatureTag", scopeEl).show();
		$(this).hide();
		return false;
	});
}

//insert string at X chars from the end
String.prototype.insertAtEnd = function (offsetFromTheEnd, string) {
	let index = this.length - offsetFromTheEnd;
	if (index > 0)
		return this.substring(0, index) + string + this.substring(index, this.length);

	return this + string;
};